library benchpress.src.web_driver_adapter;

import "package:angular2/src/core/di.dart" show bind, provide, Provider;
import "package:angular2/src/facade/async.dart" show Future;
import "package:angular2/src/facade/exceptions.dart"
    show BaseException, WrappedException;
import "package:angular2/src/facade/collection.dart" show Map;

/**
 * A WebDriverAdapter bridges API differences between different WebDriver clients,
 * e.g. JS vs Dart Async vs Dart Sync webdriver.
 * Needs one implementation for every supported WebDriver client.
 */
abstract class WebDriverAdapter {
  static List<Provider> bindTo(delegateToken) {
    return [
      bind(WebDriverAdapter).toFactory((delegate) => delegate, [delegateToken])
    ];
  }

  Future<dynamic> waitFor(Function callback) {
    throw new BaseException("NYI");
  }

  Future<dynamic> executeScript(String script) {
    throw new BaseException("NYI");
  }

  Future<dynamic> executeAsyncScript(String script) {
    throw new BaseException("NYI");
  }

  Future<Map<String, dynamic>> capabilities() {
    throw new BaseException("NYI");
  }

  Future<List<dynamic>> logs(String type) {
    throw new BaseException("NYI");
  }
}
